#EXEC OBJ LOAD FILE="ImportOBJs\MaricaFlatoutModels.ukx" Package=ProjectMaricaFlatoutCar
#EXEC OBJ LOAD FILE="ImportOBJs\MaricaFlatoutTex.utx" Package=ProjectMaricaFlatoutCar
#EXEC OBJ LOAD FILE="ImportOBJs\MaricaFlatoutSounds.uax" Package=ProjectMaricaFlatoutCar
#EXEC OBJ LOAD FILE="ImportOBJs\MaricaFlatoutCollision.usx" Package=ProjectMaricaFlatoutCar

//=============================================================================
// FlatoutCar.uc
//
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// For: ProjectMarica FlatoutCar
// Site: http://www.set-games.ru
// Steam: http://steamcommunity.com/profiles/76561198136381284/
//
// 2016-2017.
//=============================================================================

class FlatoutCar extends NKVCar;

var() vector WheelIT1;
var() vector WheelIT2;

var() float SurfaceTypeDriftMult;
var() float TireLateralFrictionDrift;

var TireFXBase DriftDustEffect;

var MapSurfaceTypeSystem SavedMapSurfaceType;

var float InvertedUpdTime;
var byte InvertedUpdCount;
var() bool bDamageWhenInverted;

replication
{
	unreliable if( Role == ROLE_Authority )
		DriftDustEffect;
}

event VehicleUpdateParams()
{
	Super.VehicleUpdateParams();

	EngTireRight.WheelIT1 = WheelIT1;
	EngTireRight.WheelIT2 = WheelIT2;

	EngTireLeft.WheelIT1 = WheelIT1;
	EngTireLeft.WheelIT2 = WheelIT2;

	SteerTireRight.WheelIT1 = WheelIT1;
	SteerTireRight.WheelIT2 = WheelIT2;

	SteerTireLeft.WheelIT1 = WheelIT1;
	SteerTireLeft.WheelIT2 = WheelIT2;
}

state VehicleDriving
{
	function DrivingSimulation(float DesiredSpeed)
	{
		local float ForwardSpeed , RotSteer , RotAng , Torque;
		local float fDriftFactor, fDriftSpeedFactor, fTireLateralFriction;
		local vector worldForward;
		local float fMaxBrakeTorque;
		local rotator rSteerRotDir;
		local vector vSteerRotDir;
		local float fSteerRotDir;

		worldForward = vect(1,0,0) >> Rotation;
		ForwardSpeed = Velocity Dot worldForward;
		WheelSpinSpeed = (EngTireRight.SpinSpeed + EngTireLeft.SpinSpeed)/2;

		if(Driver == None && !bAIControlled)
			return;

		if(DesiredSpeed > 1100)
			RotAng = 1.0;
		else RotAng = 1.5;

		if( ForwardSpeed < -1.0 )
		{
			EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
		}
		else
		{
			if( EngTireRight.WheelJoint.KMaxSpeed != VehicleMaxSpeed )
			{
				EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
			}
		}

		Torque = EngineSimulation(Throttle) * 0.5;
		EngineModel(ForwardSpeed, Torque);

		if(bCarIsRTWD)
		{
			EngTireRight.WheelJoint.KMotorTorque = Torque;
			EngTireLeft.WheelJoint.KMotorTorque = Torque;
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}
		if(bCarIsFTWD)
		{
			SteerTireRight.WheelJoint.KMotorTorque = Torque;
			SteerTireLeft.WheelJoint.KMotorTorque = Torque;
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}

		RotSteer = Steering * MaxSteerAngle * RotAng;
		SteerTireRight.WheelJoint.KSteerAngle = RotSteer;
		SteerTireLeft.WheelJoint.KSteerAngle = RotSteer;

		/// TRANSMISSION / BRAKING
		if( Throttle > 0.01 )
		{
			if( ForwardSpeed < -StopThreshold )
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
			}
			else
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
			}
		}
		else if( Throttle < -0.01 )
		{
			if( ForwardSpeed > StopThreshold )
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = true;
			}
			else
			{
				Gear = -1;
				OutputBrake = false;
				IsDriving = false;
			}
		}
		else
		{
			Gear = 0;
			IsDriving = false;
			if( abs(ForwardSpeed) < StopThreshold )
				OutputBrake = true;
			else OutputBrake = false;
		}

		fMaxBrakeTorque = MaxBrakeTorque;
		if( DriverController != None && DriverController.bWantsToSkip > 0 && !OutputBrake )
		{
			fMaxBrakeTorque = MaxBrakeTorque*0.5;
			//Gear = 0;
			OutputBrake = true;
			//IsDriving = true;
		}

		if((ForwardSpeed > HandbrakeThresh || OutputHandbrakeOn == true) 
				&& Abs(Steering) > 0.01 
				&& OutputBrake == true)
			OutputHandbrakeOn = true;
		else
			OutputHandbrakeOn = false;

		if(OutputBrake)
		{
			EngTireRight.WheelJoint.KBraking = fMaxBrakeTorque;
			EngTireLeft.WheelJoint.KBraking = fMaxBrakeTorque;
			SteerTireRight.WheelJoint.KBraking = fMaxBrakeTorque;
			SteerTireLeft.WheelJoint.KBraking = fMaxBrakeTorque;
		}
		else
		{
			EngTireRight.WheelJoint.KBraking = 0.0;
			EngTireLeft.WheelJoint.KBraking = 0.0;
			SteerTireRight.WheelJoint.KBraking = 0.0;
			SteerTireLeft.WheelJoint.KBraking = 0.0;
		}

		if(OutputHandbrakeOn == true)
		{
			SteerTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
			SteerTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
			EngTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
			EngTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
		}
		else
		{
			SteerTireRight.LateralSlip = TireLateralSlip;
			SteerTireLeft.LateralSlip = TireLateralSlip;
			EngTireRight.LateralSlip = TireLateralSlip;
			EngTireLeft.LateralSlip = TireLateralSlip;
		}

		fDriftSpeedFactor = FClamp(abs(ForwardSpeed)/33,45,100);
		fDriftFactor = FClamp(abs(abs(ForwardSpeed)-VSize(Velocity))/fDriftSpeedFactor,0,1);
		xUpdateDriftDust(fDriftFactor);
		rSteerRotDir.Yaw = Rotation.Yaw+((SteeringAngle/MaxSteerAngle)*6000);
		vSteerRotDir = vector(rSteerRotDir);
		fSteerRotDir = abs(Normal(vSteerRotDir) dot Normal(Velocity));
		fDriftFactor *= fSteerRotDir;
		fTireLateralFriction = TireLateralFriction + (TireLateralFrictionDrift*SurfaceTypeDriftMult*fDriftFactor);
		///Driver.ClientMessage(fSteerRotDir@fTireLateralFriction);
		///Driver.ClientMessage(VSize(Velocity)@ForwardSpeed@(ForwardSpeed-VSize(Velocity))@fDriftSpeedFactor@fDriftFactor@fTireLateralFriction);

		SteerTireRight.LateralFriction = fTireLateralFriction;
		SteerTireLeft.LateralFriction = fTireLateralFriction;
		EngTireRight.LateralFriction = fTireLateralFriction;
		EngTireLeft.LateralFriction = fTireLateralFriction;

		KWake();
		UpdateRearLight();
	}
}

function UpdatePhysParameters()
{
	Super.UpdatePhysParameters();
	/*if( !IsInAir() )
	{
		if( Driver != None )
			Driver.ClientMessage(self$" on ground");
	}
	else
	{
		if( Driver != None )
			Driver.ClientMessage(self$" NOT on ground");
	}*/
}

simulated function xUpdateDriftDustEffect()
{
	local vector vPos;
	if( DriftDustEffect != None )
	{
		///DriftDustEffect.Destroy();
		DriftDustEffect.ForceSelfDestroy();
		DriftDustEffect = None;
	}
	if( DriftDustEffect == None && EngWIRight != None && EngWIRight.Effect != None )
	{
		vPos.X = WheelEngAlong;
		vPos.Z = WheelVert*0.5;
		DriftDustEffect = spawn(EngWIRight.Effect.class,self,,Location+(vPos>>Rotation));
		DriftDustEffect.Emitters[0].ParticlesPerSecond = 0;
		//DriftDustEffect.Emitters[0].SizeScale[0].RelativeSize = 1.5;
		//DriftDustEffect.Emitters[0].SizeScale[1].RelativeSize = 3.5;
		DriftDustEffect.xUpdateRelativeSize();
		DriftDustEffect.bNETRelativeSizeUpd = true;
		DriftDustEffect.SetBase(Self);
	}
}

simulated function xSurfaceTypeReseted()
{
	xSurfaceTypeChanged(SavedMapSurfaceType);
}

simulated function xSurfaceTypeChanged(MapSurfaceTypeSystem eType)
{
	local float fSurfaceSlip;

	if( MapSurfaceType == eType )
		return;
	SavedMapSurfaceType = MapSurfaceType;

	if( eType == MST_Sand )
		SurfaceTypeDriftMult = 3;
	else if( eType == MST_Grass )
		SurfaceTypeDriftMult = 4;
	else if( eType == MST_Snow || eType == MST_Dirt )
		SurfaceTypeDriftMult = 5;
	else if( eType == MST_Ice )
		SurfaceTypeDriftMult = 6;
	else SurfaceTypeDriftMult = default.SurfaceTypeDriftMult;

	if( SurfaceTypeDriftMult > default.SurfaceTypeDriftMult )
		fSurfaceSlip = 0.03 * SurfaceTypeDriftMult;
	TireRollFriction = default.TireRollFriction - fSurfaceSlip;
	TireLateralFriction = default.TireLateralFriction - fSurfaceSlip;

	MapSurfaceType = eType;
	EngWIRight.UpdateDustEffect(self);
	EngWILeft.UpdateDustEffect(self);
	SteerWIRight.UpdateDustEffect(self);
	SteerWILeft.UpdateDustEffect(self);
	xUpdateDriftDustEffect();

	///if( Driver != None )
	///	Driver.ClientMessage("surface changed"@MapSurfaceType@SurfaceTypeDriftMult@fSurfaceSlip);
	///log("surface changed"@MapSurfaceType@SurfaceTypeDriftMult@fSurfaceSlip);
}

simulated function xUpdateDriftDust(float fDriftFactor)
{
	local float SpritePPS;
	local float fGroundSlipVel;

	if( DriftDustEffect != None && EngTireRight != None )
	{
		fGroundSlipVel = EngTireRight.GroundSlipVel/5;
		if( !IsInAir() && ((fDriftFactor > 0.1 && mySpeed > 800) || (fGroundSlipVel > DustSlipThresh && mySpeed > 400)) )
		{
			fDriftFactor *= 3;
			SpritePPS = FClamp(16*DustSlipRate*DustSlipThresh*(fDriftFactor+fGroundSlipVel),0,35);
			///Driver.ClientMessage(fGroundSlipVel@SpritePPS@SpritePPS*3);

			DriftDustEffect.Emitters[0].ParticlesPerSecond = SpritePPS;
			DriftDustEffect.Emitters[0].InitialParticlesPerSecond = SpritePPS;
			DriftDustEffect.Emitters[0].AllParticlesDead = False;
			DriftDustEffect.SoundVolume = FClamp(SpritePPS*3,0,SlipVolume);
		}
		else
		{
			DriftDustEffect.Emitters[0].ParticlesPerSecond = 0;
			DriftDustEffect.Emitters[0].InitialParticlesPerSecond = 0;
			DriftDustEffect.SoundVolume = 0;
		}
	}
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();
	xUpdateDriftDustEffect();
	if( MapSurfaceType != default.MapSurfaceType )
		xSurfaceTypeChanged(MapSurfaceType);
}

simulated function Destroyed()
{
	if( DriftDustEffect != None )
		DriftDustEffect.Destroy();
	Super.Destroyed();
}

simulated function SearchWeatherController()
{
	local NKSurfaceTypeStartInfo WeatherController;
	foreach AllActors(class'NKSurfaceTypeStartInfo',WeatherController)
	{
		if( WeatherController != None )
		{
			WeatherController.TakeSurfaceType(self);
			break;
		}
	}
}

simulated function Scrape(Actor Other, vector Pos)
{
	if( Other != None )
		Super.Scrape(Other, Pos);
}

simulated function ContinuousUpdate()
{
	Super.ContinuousUpdate();
	if( bDamageWhenInverted )
	{
		if( Level.TimeSeconds > InvertedUpdTime + 0.8 )
		{
			if( Health > 0 )
			{
				if( IsInverted() && mySpeed <= 10 )
					InvertedUpdCount++;
				else InvertedUpdCount = 0;
				if( InvertedUpdCount >= 5 )
				{
					if( !bImpactTimer )
					{
						ImpactInfo.Other = None;
						ImpactInfo.Speed = 10000;
						ImpactInfo.Pos = Location;
						ImpactInfo.MyVelocity = VSize(Velocity);
						ImpactInfo.OtherVelocity = 0;
						SetTimer(0.1,false);
						bImpactTimer = true;
					}
				}
			}
			InvertedUpdTime = Level.TimeSeconds;
		}
	}
}

defaultproperties
{
	bDamageWhenInverted=true

	SuspStiffness=200.0
	SuspDamping=6.0
	SuspHighLimit=0.4
	SuspLowLimit=-0.4

	TireRollFriction=1.0
	TireLateralFriction=1.0
	TireLateralFrictionDrift=-0.1
	SurfaceTypeDriftMult=2

	bCarIsFTWD=true
	ChassisMass=5.0
	SteerSpeed=12000.0
	VehicleMaxSpeed=15155325.0
	TireMass=3.0
	TorqueCurve=(Points=((InVal=0.0,OutVal=120.0),(InVal=650000.0,OutVal=50.0),(InVal=900000.0))

	//WheelIT1=(X=0.01,Y=0.0,Z=0.01)
	//WheelIT2=(X=0.01,Y=0.0,Z=0.0)

	 Tag="SpeedEvil"
	 ShadowZ=0.0

	 EngineVolume=55
	 //SteerSpeed=31500.000000
	 MaxBrakeTorque=45
	 WheelRadius=37.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=0.7

     EngStartUpSound=None
     EngStopSound=None
     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_02'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_02'
	 ImpactSoundsMin(0)=Sound'ProjectMaricaFlatoutCar.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'ProjectMaricaCore.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'ProjectMaricaCore.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'ProjectMaricaFlatoutCar.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'ProjectMaricaFlatoutCar.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'ProjectMaricaFlatoutCar.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'ProjectMaricaFlatoutCar.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=300

	 CameraDist=13.0
	 Description="SpeedEvil from Flatout by BugBear."
	 DestroyBones(0)="Bonnet"
	 DestroyBones(1)="Windows"
	 DestroyBones(2)="Attachment"
	 bAllowBrokenWheels=True
	 bSpawnBrokenDetail=True

	 bFixedFPCamera=false
	 FrontLightMaterial=Texture'GenFX.LensFlar.6'
	 RearLightMaterial=Texture'GenFX.LensFlar.softlens2'
	 ///RearLightMaterial=Texture'ProjectMaricaFlatoutCar.flares'
	 RearLightStopScale=0.200000
	 RearLightScale=0.100000
	 FrontLightScale=0.200000
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     FrontLightPos(0)=(X=216.000000,Y=50.000000,Z=-2.000000)
     FrontLightPos(1)=(X=216.000000,Y=-50.000000,Z=-2.000000)
     FrontLightPos(2)=(X=216.000000,Y=68.000000,Z=-2.000000)
     FrontLightPos(3)=(X=216.000000,Y=-68.000000,Z=-2.000000)
     FrontLightPos(4)=(X=216.000000,Y=28.500000,Z=-27.500000)
     FrontLightPos(5)=(X=216.000000,Y=-28.500000,Z=-27.500000)
     RearLightPos(0)=(X=-220.000000,Y=55.000000,Z=10.000000)
     RearLightPos(1)=(X=-220.000000,Y=-55.000000,Z=10.000000)

     ExhaustClass=Class'CarExhaust'
	 ExhaustPos(0)=(X=-88.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(1)=(X=-88.000000,Y=-86.000000,Z=-45.000000)
	 ExhaustPos(2)=(X=-65.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(3)=(X=-65.000000,Y=-86.000000,Z=-45.000000)

	 DecoSystem=DS_Bones
	 ExhaustBones(0)="Exhaust"
	 ExhaustBones(1)="Exhaust01"
	 SteerWheelBone="SteerWheel"

	 FrontLightBones(0)="FrontLight00"
	 FrontLightBones(1)="FrontLight01"

	 RearLightBones(0)="RearLight00"
	 RearLightBones(1)="RearLight01"

	 bUpdateSusp=True
	 RSuspHighLimit=17.000000
	 RSuspLowLimit=-17.000000
     ExitPos=(Y=-140.000000,Z=15.000000)
     ExitPosPassenger=(Y=140.000000,Z=15.000000)
     EnterPos=(Y=145.000000,Z=-5.000000)
	 WheelFrontRight="RightFrontTIRE"
	 WheelFrontLeft="LeftFrontTIRE"
	 WheelRearRight="RightRearTIRE"
	 WheelRearLeft="LeftRearTIRE"
	 StrutFrontRight=""
	 StrutFrontLeft=""
	 StrutRearRight=""
	 StrutRearLeft=""
	IntCollisionSize=(X=1,Y=1,Z=1)
	IntCollisionModel=StaticMesh'ProjectMaricaFlatoutCar.pm_speedevil_intcoll'
	IntCollisionClass=Class'IntCollisionActor'

	LightProjectorClass=None

     bCarIsRTWD=True
     MaxSpeedAccel=30.000000
	 VehicleMaxSpeed=655325.000000
     MaxSteerAngle=3500.000000

	 bCanSeeDriver=True
	 DriverPos=(X=-29.000000,Y=-36.000000,Z=-22.000000)
	 FPDriverViewPos=(X=-28.000000,Y=-39.000000,Z=40.000000)
	 FPPassengerViewPos=(X=-28.000000,Y=39.000000,Z=40.000000)
	 PassengerPos=(X=-29.000000,Y=36.000000,Z=-22.000000)

	 RadiatorPos=(X=125.000000,Y=0.000000,Z=25.000000)
     VehicleFireClass=Class'VehicleFireEmitter'

	BrokenSkins(0)=Texture'ProjectMaricaFlatoutCar.skin1'

	 bDamagableWheels=True
	 WheelsHealth=50
	 WheelsMaxHealth=100
	 UnloadWheelHeight=0.006
	 bSparkWheels=True
     EngTireClass=Class'FlatoutCarTire'
     SteerTireClass=Class'FlatoutCarTire'
     WheelSteerAlong=139.000000
     WheelSteerAcross=73.000000
     WheelEngAlong=-141.500000
     WheelEngAcross=73.000000
     WheelVert=-37.800000
     //SuspStiffness=200.000000
     //SuspDamping=5.000000
     //SuspHighLimit=0.400000
     //SuspLowLimit=-0.400000
     //TireRollFriction=1.500000
     //TireLateralFriction=2.000000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000

	 SlipVolume=20.0
	 EnginePitch=50.0
     EngineSound=Sound'ProjectMaricaFlatoutCar.4_1_onmid_ex'
     ImpactSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactSounds(1)=Sound'ProjectMaricaCore.ut_impact_02'

	StaticMesh=StaticMesh'ProjectMaricaFlatoutCar.pm_speedevil_kcoll'
	Mesh=SkeletalMesh'ProjectMaricaFlatoutCar.pm_speedevil_model01'

	bRandomSkin=false
	SkinIndex=0
	Skins(0)=Texture'ProjectMaricaFlatoutCar.skin1'
	Skins(1)=Texture'ProjectMaricaFlatoutCar.common'
	Skins(2)=Texture'ProjectMaricaFlatoutCar.interior'
	Skins(3)=Shader'ben_skins.env_maps.Translucent'
	Skins(4)=Shader'ben_skins.env_maps.Translucent'
	Skins(5)=Texture'ProjectMaricaFlatoutCar.tire_01'
	Skins(6)=Texture'ProjectMaricaFlatoutCar.common'
	Skins(7)=Texture'ProjectMaricaFlatoutCar.dashboard'
	Skins(8)=Texture'ProjectMaricaFlatoutCar.shock'
	Skins(9)=Texture'ProjectMaricaFlatoutCar.common'

	OtherSkins(0)=Texture'ProjectMaricaFlatoutCar.skin1'

     Begin Object Class=KarmaParamsRBFull Name=SpeedEvilPhysics
         //KInertiaTensor(0)=20.000000
         //KInertiaTensor(3)=30.000000
         //KInertiaTensor(5)=58.000000
         KInertiaTensor(0)=4.000000
         KInertiaTensor(3)=25.000000
         KInertiaTensor(5)=45.000000
         KCOMOffset=(X=0.5,Z=-1.5)
		 KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="SpeedEvilPhysics"
     End Object
     KParams=KarmaParamsRBFull'SpeedEvilPhysics'
}